/*
 * Includes
 */
#include <linux/kernel.h>	/* printk() */
#include <linux/module.h>	/* modules */
#include <linux/fs.h>           /* file_operations */
#include <linux/device.h>	/* class */

MODULE_DESCRIPTION("mydriver1");
MODULE_AUTHOR("Pierre Ficheux, Open Wide");
MODULE_LICENSE("GPL");

/*
 * Arguments
 */
static int major = 0; /* Major number */

module_param(major, int, 0644);
MODULE_PARM_DESC(major, "Static major number (none = dynamic)");

// Driver class in /sys
static struct class *mydriver1_class;

/*
 * File operations
 */
static ssize_t mydriver1_read(struct file *file, char *buf, size_t count, loff_t *ppos)
{
	printk(KERN_INFO "mydriver1: read()\n");

	return count;
}

static ssize_t mydriver1_write(struct file *file, const char *buf, size_t count, loff_t *ppos)
{
	printk(KERN_INFO "mydriver1: write()\n");

	return count;
}

static int mydriver1_open(struct inode *inode, struct file *file)
{
	printk(KERN_INFO "mydriver1: open()\n");

	return 0;
}

static int mydriver1_release(struct inode *inode, struct file *file)
{
	printk(KERN_INFO "mydriver1: release()\n");

	return 0;
}

static struct file_operations mydriver1_fops = {
	.owner =	THIS_MODULE,
	.read =		mydriver1_read,
	.write =	mydriver1_write,
	.open =		mydriver1_open,
	.release =	mydriver1_release,
};

/*
 * Init and Exit
 */
static int __init mydriver1_init(void)
{
	int ret;

	ret = register_chrdev(major, "mydriver1", &mydriver1_fops);
	if (ret < 0) {
		printk(KERN_WARNING "mydriver1: unable to get a major\n");

		return ret;
	}

	if (major == 0)
		major = ret; /* dynamic value */

	// Class creation in /sys/class
	mydriver1_class = class_create (THIS_MODULE, "myclass");
	if (!mydriver1_class)
	  printk(KERN_ERR "mydriver1: failed to create class\n");
	else
	  // Create entry in /dev
	  device_create(mydriver1_class, NULL, MKDEV(major, 0), NULL, "mydriver1");

	printk(KERN_INFO "mydriver1: successfully loaded with major %d\n", major);

	return 0;
}

static void __exit mydriver1_exit(void)
{
	unregister_chrdev(major, "mydriver1"); 
	// Remove device entry + class
	device_destroy(mydriver1_class, MKDEV(major, 0));
	class_destroy (mydriver1_class);
	printk(KERN_INFO "mydriver1: successfully unloaded\n");
}

/*
 * Module entry points
 */
module_init(mydriver1_init);
module_exit(mydriver1_exit);
