/*
 * Includes
 */
#include <linux/kernel.h>	/* printk() */
#include <linux/module.h>	/* modules */
#include <linux/init.h>		/* module_{init,exit}() */
#include <linux/fs.h>           /* file_operations */
#include <linux/miscdevice.h>   /* misc driver interface */

MODULE_DESCRIPTION("mydriver2");
MODULE_AUTHOR("Stelian Pop/Pierre Ficheux, Open Wide");
MODULE_LICENSE("GPL");

static struct miscdevice mymisc; /* Misc device handler */

/*
 * File operations
 */
static ssize_t mydriver2_read(struct file *file, char *buf, size_t count, loff_t *ppos)
{
  printk(KERN_INFO "mydriver2: read(), ppos= %d\n", (int)*ppos);

  return count;
}

static ssize_t mydriver2_write(struct file *file, const char *buf, size_t count, loff_t *ppos)
{
  printk(KERN_INFO "mydriver2: write(), ppos= %d %s\n", (int)*ppos, buf);

  return count;
}

static int mydriver2_open(struct inode *inode, struct file *file)
{
  printk(KERN_INFO "mydriver2: open()\n");

  return 0;
}

static int mydriver2_release(struct inode *inode, struct file *file)
{
  printk(KERN_INFO "mydriver2: release()\n");

  return 0;
}

static struct file_operations mydriver2_fops = {
  .owner   =	THIS_MODULE,
  .read    =	mydriver2_read,
  .write   =	mydriver2_write,
  .open    =	mydriver2_open,
  .release =	mydriver2_release,
};

/*
 * Init and Exit
 */
static int __init mydriver2_init(void)
{
  int ret;

  mymisc.minor = MISC_DYNAMIC_MINOR;
  mymisc.name = "mydriver2";
  mymisc.fops = &mydriver2_fops;

  ret = misc_register(&mymisc);

  if (ret < 0) {
    printk(KERN_WARNING "mydriver2: unable to get a dynamic minor\n");
    return ret;
  }

  return 0;
}

static void __exit mydriver2_exit(void)
{
  misc_deregister(&mymisc);
  printk(KERN_INFO "mydriver2: successfully unloaded\n");
}

/*
 * Module entry points
 */
module_init(mydriver2_init);
module_exit(mydriver2_exit);
