/*
 * Includes
 */
#include <linux/kernel.h>	/* printk() */
#include <linux/module.h>	/* modules */
#include <linux/init.h>		/* module_{init,exit}() */
#include <linux/fs.h>           /* file_operations */
#include <asm/uaccess.h>	/* copy_{from,to}_user() */
#include <linux/miscdevice.h>   /* misc driver interface */

MODULE_DESCRIPTION("mydriver5");
MODULE_AUTHOR("Stelian Pop/Pierre Ficheux, Open Wide");
MODULE_LICENSE("GPL");

/* ioctl cmds */
#define SET_MODE 0
#define GET_MODE 1

/* ioctl valid args */
#define MODE1    1
#define MODE2    2


/*
 * Global variables
 */

static int my_mode = MODE1;
static struct miscdevice mymisc; /* Misc device handler */

/*
 * File operations
 */
static ssize_t mydriver5_read(struct file *file, char *buf, size_t count, loff_t *ppos)
{
  printk(KERN_INFO "mydriver5: read()\n");
  
  return count;
}

static ssize_t mydriver5_write(struct file *file, const char *buf, size_t count, loff_t *ppos)
{
  printk(KERN_INFO "mydriver1: write()\n");

  return count;
}

static int mydriver5_open(struct inode *inode, struct file *file)
{
  printk(KERN_INFO "mydriver5: open()\n");

  return 0;
}

static int mydriver5_release(struct inode *inode, struct file *file)
{
  printk(KERN_INFO "mydriver5: release()\n");

  return 0;
}

static int mydriver5_ioctl(struct file *file, unsigned int cmd, unsigned long arg)
//static int mydriver5_ioctl(struct inode *inode, struct file *file, 
//			   unsigned int cmd, unsigned long arg)
{
  printk(KERN_DEBUG "mydriver5: ioctl()\n");

  switch (cmd) {
  case SET_MODE :
    switch (arg) {
    case MODE1 :
      my_mode = MODE1;
      break;

    case MODE2 :
      my_mode = MODE2;
      break;

    default :
      printk(KERN_WARNING "mydriver5: arg %x unsupported in the SET_MODE ioctl command\n", (int)arg);
      
      return -EINVAL;
    }
    break;

  case GET_MODE: /* Send my_mode value to user space */
    if (put_user(my_mode, (long __user *)arg)) {
      printk (KERN_WARNING "mydriver5: put_user error\n");
      return -EFAULT;
    }
	
    break;
  
  default :
    printk(KERN_WARNING "mydriver5: 0x%x unsupported ioctl command\n", cmd);
    
    return -EINVAL;
  }

  return 0;

}


static struct file_operations mydriver5_fops = {
  .owner   =	THIS_MODULE,
  .read    =	mydriver5_read,
  .write   =	mydriver5_write,
  .unlocked_ioctl   =	mydriver5_ioctl, // si >= 2.6.36
  //  .ioctl   =	mydriver5_ioctl,
  .open    =	mydriver5_open,
  .release =	mydriver5_release,
};

/*
 * Init and Exit
 */
static int __init mydriver5_init(void)
{
  int ret;

  mymisc.minor = MISC_DYNAMIC_MINOR;
  mymisc.name = "mydriver5";
  mymisc.fops = &mydriver5_fops;

  ret = misc_register(&mymisc);

  if (ret < 0) {
    printk(KERN_WARNING "mydriver5: unable to get a dynamic minor\n");
    return ret;
  }

  return 0;
}

static void __exit mydriver5_exit(void)
{
  misc_deregister(&mymisc);

  printk(KERN_INFO "mydriver5: successfully unloaded\n");
}

/*
 * Module entry points
 */
module_init(mydriver5_init);
module_exit(mydriver5_exit);
