/*
 * Includes
 */
#include <linux/module.h>	/* modules */
#include <linux/kernel.h>	/* printk() */
#include <linux/fs.h>           /* file_operations */
#include <asm/io.h>
#include <asm/uaccess.h>	/* copy_{from,to}_user() */

MODULE_DESCRIPTION("Simple pp driver");
MODULE_AUTHOR("Pierre Ficheux, Open Wide");
MODULE_LICENSE("GPL");

#define GET_CNT		0
#define LPT             0x378

/*
 * Arguments
 */
static int major = 0; /* Major number */
static unsigned long nwrite;

module_param(major, int, 0644);
MODULE_PARM_DESC(major, "Static major number (none = dynamic)");

#define BUF_SIZE 10

static char buffer[BUF_SIZE];

/*
 * File operations
 */
static ssize_t pp_write(struct file *file, const char *buf, size_t count, loff_t *ppos)
{
  size_t real;
  register int i;

  real = min((size_t)BUF_SIZE, count);

  if (real)
    if (copy_from_user(buffer, buf, real))
      return -EFAULT;

  /* Ecriture sur le port // */
  for (i = 0 ; i < real ; i++) {	
//	printk (KERN_INFO "writing %c %x to LPT\n", (buffer[i] >= ' ' ? buffer[i] : '.'), buffer[i]);
  	outb (buffer[i], LPT);
  	nwrite++;
  }


  return real;
}

static int pp_ioctl(struct inode *inode, struct file *file, 
			   unsigned int cmd, unsigned long arg)
{

  switch (cmd) {
  case GET_CNT: /* Send nwrite value to user space */
    if (copy_to_user((void*)arg, &nwrite, sizeof(unsigned long))) {
      printk(KERN_WARNING "pp: copy_to_user error\n");
      return -EFAULT;
    }
  
    printk(KERN_DEBUG "pp: ioctl() => %lu\n", nwrite);
    break;
  

  default :
    printk(KERN_WARNING "pp: 0x%x unsupported ioctl command\n", cmd);
    
    return -EINVAL;
  }

  return 0;

}
static int pp_open(struct inode *inode, struct file *file)
{
	printk(KERN_INFO "pp: open()\n");
	nwrite = 0;

	return 0;
}

static int pp_release(struct inode *inode, struct file *file)
{
	printk(KERN_INFO "pp: release()\n");

	return 0;
}

static struct file_operations pp_fops = {
	.owner =	THIS_MODULE,
	.write =	pp_write,
	.open =		pp_open,
	.release =	pp_release,
	.ioctl =	pp_ioctl,
};

/*
 * Init and Exit
 */
static int __init pp_init(void)
{
	int ret;

	ret = register_chrdev(major, "pp", &pp_fops);
	if (ret < 0) {
		printk(KERN_WARNING "pp: unable to get a major\n");

		return ret;
	}

	if (major == 0)
		major = ret; /* dynamic value */

	printk(KERN_INFO "pp: successfully loaded with major %d\n", major);

	return 0;
}

static void __exit pp_exit(void)
{
	unregister_chrdev(major, "pp"); 
	printk(KERN_INFO "pp: successfully unloaded\n");
}

/*
 * Module entry points
 */
module_init(pp_init);
module_exit(pp_exit);
