#include <linux/module.h>
#include <linux/kernel.h>
#include <asm/io.h>
#include <linux/timer.h>
#include <linux/proc_fs.h>	/* Pour /proc */

MODULE_DESCRIPTION("Square module");
MODULE_AUTHOR("Pierre Ficheux, Open Wide");
MODULE_LICENSE("GPL");

#define PERIOD         HZ/20   /* 50 ms = 1000 / 20 s */
#define LPT            0x378

static struct timer_list timer;
static int timer_cnt;
static int nibl=0x01;

/* Appele sur accs  /proc/square */
static int square_get_info(char *buf, char **start, off_t off, int count, int *eof, void *data)
{
  int len = 0;

  len += sprintf(buf + len, "timer_cnt= %d\n", timer_cnt);
  *eof = 1;
  
  return len;
}

static void timer_handler(unsigned long arg) 
{
  timer_cnt++;

  /* Ecriture sur le port // */
  outb(nibl, LPT);
  nibl = ~nibl;

  mod_timer(&timer, jiffies + PERIOD);
}

static int __init square_init(void)
{
  struct proc_dir_entry *proc_entry;

  printk(KERN_INFO "Square module, loading!\n");

  /* Entre /proc */
  proc_entry = create_proc_read_entry("square", 0, NULL, square_get_info, NULL);
  if (!proc_entry) {
    printk(KERN_ERR "Can't create proc entry for square\n");
    return -EAGAIN;
  }

  /* Initialise le timer */
  init_timer(&timer);
  timer.function = timer_handler;
  timer.data = 0;
  mod_timer(&timer, jiffies + PERIOD);


  return 0;
}

static void __exit square_exit(void)
{
  del_timer_sync(&timer);
  remove_proc_entry ("square", NULL);
  
  printk(KERN_INFO "Square module, exiting!\n");
}

module_init(square_init);
module_exit(square_exit);
