/*
 * Includes
 */
#include <linux/kernel.h>	/* printk() */
#include <linux/module.h>	/* modules */
#include <linux/init.h>		/* module_{init,exit}() */
#include <linux/slab.h>		/* kmalloc()/kfree() */
#include <linux/pci.h>		/* pci_*() */
#include <linux/pci_ids.h>	/* pci idents */
#include <linux/list.h>		/* list_*() */
#include <asm/uaccess.h>	/* copy_{from,to}_user() */
#include <linux/fs.h>		/* file_operations */
#include <linux/interrupt.h>	/* request_irq etc */
#include <linux/version.h>

MODULE_DESCRIPTION("pcidemo");
MODULE_AUTHOR("Stelian Pop/Pierre Ficheux, Open Wide");
MODULE_LICENSE("GPL");

/*
 * Supported devices
 */
// Change your Vendor-ID + Device-ID here
#define VENDOR_ID	0x1969
#define DEVICE_ID	0x1062

static struct pci_device_id pcidemo_id_table[] __devinitdata = {
  {VENDOR_ID, DEVICE_ID, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
  {0,}	/* 0 terminated list */
};
MODULE_DEVICE_TABLE(pci, pcidemo_id_table);

/*
 * PCI handling
 */
static int __devinit pcidemo_probe(struct pci_dev *dev, const struct pci_device_id *ent)
{
  printk(KERN_INFO "pcidemo: found %x:%x\n", ent->vendor, ent->device);

  return 0;
}

static void __devexit pcidemo_remove(struct pci_dev *dev)
{
  printk(KERN_INFO "pcidemo: device removed\n");
}

static struct pci_driver pcidemo_driver = {
  .name =		"pcidemo",
  .id_table =	pcidemo_id_table,
  .probe =	pcidemo_probe,		/* Init one device */
  .remove =	pcidemo_remove,		/* Remove one device */
};

/*
 * Init and Exit
 */
static int __init pcidemo_init(void)
{
  int ret;

  /* Register PCI driver */
  ret = pci_register_driver(&pcidemo_driver);
  if (ret < 0) {
    printk(KERN_WARNING "pcidemo: unable to register PCI driver\n");
    return ret;
  }

  return 0;
}

static void __exit pcidemo_exit(void)
{
  pci_unregister_driver(&pcidemo_driver);
}

/*
 * Module entry points
 */
module_init(pcidemo_init);
module_exit(pcidemo_exit);
