#include <linux/errno.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/usb.h>

#define DRIVER_AUTHOR "Pierre Ficheux, pierre@ficheux.org"
#define DRIVER_DESC "Panic Button driver: version 1.5"

// Device definition
#define VENDOR_ID	0x1130
#define PRODUCT_ID	0x0202

struct usb_panicb {
	struct usb_device *	udev;
	unsigned int		button;
};

/* Table of devices that work with this driver */
static struct usb_device_id id_table [] = {
	{ USB_DEVICE(VENDOR_ID, PRODUCT_ID) },
	{ },
};
MODULE_DEVICE_TABLE (usb, id_table);

//
// /sys interface
//
static ssize_t show_button(struct device *dev, struct device_attribute *attr, char *buf)
{
  struct usb_interface *intf = to_usb_interface(dev); 
  struct usb_panicb *panicb_dev = usb_get_intfdata(intf);

  // To be continued
  //get_panicb_button_status (panicb_dev);
  printk (KERN_INFO "%s\n", __FUNCTION__);

  return sprintf(buf, "%d\n", panicb_dev->button); 
}

static ssize_t set_button(struct device *dev, struct device_attribute *attr, const char *buf, size_t count)
{
  printk (KERN_INFO "%s\n", __FUNCTION__);
  return count;
}

// Define "button" entry
static DEVICE_ATTR(button, S_IWUGO | S_IRUGO, show_button, set_button);


static int panicb_probe (struct usb_interface *interface, const struct usb_device_id *id)
{
  struct usb_device *udev = interface_to_usbdev (interface);
  struct usb_panicb *panicb_dev;

  printk (KERN_INFO "panicb_probe()\n");

  // Fill private structure and save it
  panicb_dev = kmalloc (sizeof(struct usb_panicb), GFP_KERNEL);
  if (panicb_dev == NULL) {
    dev_err (&interface->dev, "Out of memory\n");
    return -ENOMEM;
  }

  memset (panicb_dev, 0, sizeof (*panicb_dev));
  panicb_dev->udev = usb_get_dev(udev);
  panicb_dev->button = 0;
  usb_set_intfdata (interface, panicb_dev);

  device_create_file(&interface->dev, &dev_attr_button);

  return 0;
}

static void panicb_disconnect(struct usb_interface *interface)
{
  struct usb_panicb *dev;

  printk (KERN_INFO "panicb_disconnect()\n");

  dev = usb_get_intfdata (interface);
  usb_set_intfdata (interface, NULL);

  device_remove_file(&interface->dev, &dev_attr_button);

  // Release device
  usb_put_dev(dev->udev);

  kfree(dev);
}

static struct usb_driver panicb_driver = {
	.name =		"panicb",
	.probe =	panicb_probe,
	.disconnect =	panicb_disconnect,
	.id_table =	id_table,
};

static int __init usb_panicb_init(void)
{
  return usb_register(&panicb_driver);
}

static void __exit usb_panicb_exit(void)
{
  usb_deregister(&panicb_driver);
}


module_init (usb_panicb_init);
module_exit (usb_panicb_exit);

MODULE_AUTHOR(DRIVER_AUTHOR);
MODULE_DESCRIPTION(DRIVER_DESC);
MODULE_LICENSE("GPL");
